extension DateTimeExtensions on DateTime {

  Duration getTimeAsDuration() => Duration(hours: hour, minutes: minute, seconds: second, milliseconds: millisecond, microseconds: microsecond);
  bool isBeforeOrEqual(DateTime other, {bool compareTime = true, bool compareSeconds = true, bool compareMillis = false, bool compareMicros = false}) {
    DateTime date1 = this;
    DateTime date2 = other;

    if (!compareTime) {
      date1 = date1.date;
      date2 = date2.date;
    }
    else {
      date1 = date1.truncateTime(seconds: !compareSeconds, millis: !compareMillis, micros: !compareMicros);
      date2 = date2.truncateTime(seconds: !compareSeconds, millis: !compareMillis, micros: !compareMicros);
    }

    return date1.isBefore(date2) || date1.isAtSameMomentAs(date2);
  }

  bool isEqual(DateTime other, {bool compareTime = true, bool compareSeconds = true, bool compareMillis = false, bool compareMicros = false}) {
    DateTime date1 = this;
    DateTime date2 = other;

    if (!compareTime) {
      date1 = date1.date;
      date2 = date2.date;
    }
    else {
      date1 = date1.truncateTime(seconds: !compareSeconds, millis: !compareMillis, micros: !compareMicros);
      date2 = date2.truncateTime(seconds: !compareSeconds, millis: !compareMillis, micros: !compareMicros);
    }

    return date1.isAtSameMomentAs(date2);
  }

  DateTime truncateTime({bool hours = false, bool minutes = false, bool seconds = false, bool millis = false, bool micros = false})
    => hours || minutes || seconds || millis || micros
      ? subtract(Duration(
        hours: hours ? hour : 0,
        minutes: minutes ? minute : 0,
        seconds: seconds ? second : 0,
        milliseconds: millis ? millisecond : 0,
        microseconds: micros ? microsecond : 0)
    ) : date;

  DateTime get date => DateTime(year, month, day);

  bool isSameDate(DateTime other) => date.isAtSameMomentAs(other.date);

  bool get isToday {
    final nowDate = DateTime.now();
    return year == nowDate.year && month == nowDate.month && day == nowDate.day;
  }

  String getTimeAsHoursMinutesSeconds() {
    final hoursStr = hour.toString().padLeft(2, '0');
    final minutesStr = minute.toString().padLeft(2, '0');
    final secondsStr = second.toString().padLeft(2, '0');

    return '$hoursStr:$minutesStr:$secondsStr';
  }
}